@@ -1,4 +1,4 @@
-# vim: set ts=4 sts=4 sw=4 expandtab smarttab:
+# vim: set ts=2 sts=2 sw=2 expandtab smarttab:
 #
 # This file is part of Pod-Markdown
 #
@@ -13,9 +13,9 @@ use warnings;
 
 package Pod::Markdown;
 {
-  $Pod::Markdown::VERSION = '1.401';
+  $Pod::Markdown::VERSION = '1.500';
 }
-# git description: v1.400-1-ge7a28a2
+# git description: v1.401-20-g3999377
 
 BEGIN {
   $Pod::Markdown::AUTHORITY = 'cpan:RWSTAUNER';
@@ -33,28 +33,84 @@ our %URL_PREFIXES = (
 );
 $URL_PREFIXES{perldoc} = $URL_PREFIXES{metacpan};
 
+
+# new() is provided by Pod::Parser (which calls initialize()).
+
 sub initialize {
     my $self = shift;
     $self->SUPER::initialize(@_);
 
     for my $type ( qw( perldoc man ) ){
         my $attr  = $type . '_url_prefix';
+        # Use provided argument or default alias.
         my $url = $self->{ $attr } || $type;
         # Expand alias if defined (otherwise use url as is).
         $self->{ $attr } = $URL_PREFIXES{ $url } || $url;
     }
 
+    $self->_prepare_fragment_formats;
+
     $self->_private;
     $self;
 }
 
 
-# For consistency with Pod::Simple method names.
-sub perldoc_url_prefix {
-    return $_[0]->{perldoc_url_prefix};
+my @attr = qw(
+  man_url_prefix
+  perldoc_url_prefix
+  perldoc_fragment_format
+  markdown_fragment_format
+);
+
+{
+  no strict 'refs'; ## no critic
+  foreach my $attr ( @attr ){
+    *$attr = sub { return $_[0]->{ $attr } };
+  }
 }
-sub man_url_prefix {
-    return $_[0]->{man_url_prefix};
+
+sub _prepare_fragment_formats {
+  my ($self) = @_;
+
+  foreach my $attr ( @attr ){
+    next unless $attr =~ /^(\w+)_fragment_format/;
+    my $type = $1;
+    my $format = $self->{ $attr };
+
+    # If one was provided.
+    if( $format ){
+      # If the attribute is a coderef just use it.
+      next if ref($format) eq 'CODE';
+    }
+    # Else determine a default.
+    else {
+      if( $type eq 'perldoc' ){
+        # Choose a default that matches the destination url.
+        my $target = $self->{perldoc_url_prefix};
+        foreach my $alias ( qw( metacpan sco ) ){
+          if( $target eq $URL_PREFIXES{ $alias } ){
+            $format = $alias;
+          }
+        }
+        # This seems like a reasonable fallback.
+        $format ||= 'pod_simple_xhtml';
+      }
+      else {
+        $format = $type;
+      }
+    }
+
+    # The short name should become a method name with the prefix prepended.
+    my $prefix = 'format_fragment_';
+    $format =~ s/^$prefix//;
+    die "Unknown fragment format '$format'"
+      unless $self->can($prefix . $format);
+
+    # Save it.
+    $self->{ $attr } = $format;
+  }
+
+  return;
 }
 
 sub _private {
@@ -70,6 +126,7 @@ sub _private {
     };
 }
 
+
 sub as_markdown {
     my ($parser, %args) = @_;
     my $data  = $parser->_private;
@@ -127,6 +184,7 @@ sub _clean_text {
     return wantarray ? @trimmed : join("\n", @trimmed);
 }
 
+# Backslash escape markdown characters to avoid having them interpreted.
 sub _escape {
     local $_ = $_[1];
 
@@ -142,6 +200,14 @@ sub _escape {
     return $_;
 }
 
+# Formats a header according to the given level.
+sub format_header {
+    my ($self, $level, $paragraph) = @_;
+    # TODO: put a name="" if $self->{embed_anchor_tags}; ?
+    # https://rt.cpan.org/Ticket/Display.html?id=57776
+    sprintf '%s %s', '#' x $level, $paragraph;
+}
+
 # Handles POD command paragraphs, denoted by a line beginning with C<=>.
 sub command {
     my ($parser, $command, $paragraph, $line_num) = @_;
@@ -324,8 +390,8 @@ sub interior_sequence {
     my %interiors = (
         'I' => sub { return '_'  . $_[$i] . '_'  },      # italic
         'B' => sub { return '__' . $_[$i] . '__' },      # bold
-        'C' => sub { return '`'  . $_[$i] . '`'  },      # monospace
-        'F' => sub { return '`'  . $_[$i] . '`'  },      # system path
+        'C' => \&_wrap_code_span,                        # monospace
+        'F' => \&_wrap_code_span,                        # system path
         # non-breaking space
         'S' => sub {
             (my $s = $_[$i]) =~ s/ /&nbsp;/g;
@@ -350,6 +416,7 @@ sub interior_sequence {
             return "&$charname;";
         },
         'L' => \&_resolv_link,
+        # TODO: create `a name=` if configured?
         'X' => sub { '' },
         'Z' => sub { '' },
     );
@@ -375,21 +442,9 @@ sub _resolv_link {
     if ($type eq 'url') {
         $url = $name;
     } elsif ($type eq 'man') {
-        # stolen from Pod::Simple::(X)HTML
-        my ($page, $part) = $name =~ /\A([^(]+)(?:[(](\S*)[)])?/;
-        $url = $self->man_url_prefix . ($part || 1) . '/' . ($page || $name);
+        $url = $self->format_man_url($name);
     } else {
-        if ($name) {
-            $url = $self->perldoc_url_prefix . $name;
-        }
-        if ($section){
-            # TODO: sites/pod formatters differ on how to transform the section
-            # TODO: we could do it according to specified url prefix or pod formatter
-            # TODO: or allow a coderef?
-            # TODO: (Pod::Simple::XHTML:idify() for metacpan)
-            # TODO: (Pod::Simple::HTML section_escape/unicode_escape_url/section_url_escape for s.c.o.)
-            $url .= '#' . $section;
-        }
+        $url = $self->format_perldoc_url($name, $section);
     }
 
     # if we don't know how to handle the url just print the pod back out
@@ -397,15 +452,162 @@ sub _resolv_link {
         return sprintf '%s<%s>', $cmd, $arg;
     }
 
+    # TODO: put unescaped section into link title? [a](b "c")
     return sprintf '[%s](%s)', ($text || $inferred), $url;
 }
 
-# Formats a header according to the given level.
-sub format_header {
-    my ($level, $paragraph) = @_[1,2];
-    sprintf '%s %s', '#' x $level, $paragraph;
+# A code span can be delimited by multiple backticks (and a space)
+# similar to pod codes (C<< code >>), so ensure we use a big enough
+# delimiter to not have it broken by embedded backticks.
+sub _wrap_code_span {
+  my ($self, undef, $arg) = @_;
+  my $longest = 0;
+  while( $arg =~ /([`]+)/g ){
+    my $len = length($1);
+    $longest = $len if $longest < $len;
+  }
+  my $delim = '`' x ($longest + 1);
+  my $pad = $longest > 0 ? ' ' : '';
+  return $delim . $pad . $arg . $pad . $delim;
 }
 
+
+sub format_man_url {
+    my ($self, $to) = @_;
+    my ($page, $part) = ($to =~ /^ ([^(]+) (?: \( (\S+) \) )? /x);
+    return $self->man_url_prefix . ($part || 1) . '/' . ($page || $to);
+}
+
+
+sub format_perldoc_url {
+  my ($self, $name, $section) = @_;
+
+  my $url_prefix = $self->perldoc_url_prefix;
+  my $url = '';
+
+  # If the link is to another module (external link).
+  if ($name) {
+    $url = $url_prefix . $name;
+  }
+
+  # See https://rt.cpan.org/Ticket/Display.html?id=57776
+  # for a discussion on the need to mangle the section.
+  if ($section){
+
+    my $method = $url
+      # If we already have a prefix on the url it's external.
+      ? $self->perldoc_fragment_format
+      # Else an internal link points to this markdown doc.
+      : $self->markdown_fragment_format;
+
+    $method = 'format_fragment_' . $method
+      unless ref($method);
+
+    {
+      # Set topic to enable code refs to be simple.
+      local $_ = $section;
+      $section = $self->$method($section);
+    }
+
+    $url .= '#' . $section;
+  }
+
+  return $url;
+}
+
+
+# TODO: simple, pandoc, etc?
+
+sub format_fragment_markdown {
+  my ($self, $section) = @_;
+
+  # If this is an internal link (to another section in this doc)
+  # we can't be sure what the heading id's will look like
+  # (it depends on what is rendering the markdown to html)
+  # but we can try to follow popular conventions.
+
+  # http://johnmacfarlane.net/pandoc/demo/example9/pandocs-markdown.html#header-identifiers-in-html-latex-and-context
+  #$section =~ s/(?![-_.])[[:punct:]]//g;
+  #$section =~ s/\s+/-/g;
+  $section =~ s/\W+/-/g;
+  $section =~ s/-+$//;
+  $section =~ s/^-+//;
+  $section = lc $section;
+  #$section =~ s/^[^a-z]+//;
+  $section ||= 'section';
+
+  return $section;
+}
+
+
+{
+  # From Pod::Simple::XHTML 3.28.
+  # The strings gets passed through encode_entities() before idify().
+  # If we don't do it here the substitutions below won't operate consistently.
+
+  # encode_entities {
+    my %entities = (
+      q{>} => 'gt',
+      q{<} => 'lt',
+      q{'} => '#39',
+      q{"} => 'quot',
+      q{&} => 'amp',
+    );
+
+    my
+      $ents = join '', keys %entities;
+  # }
+
+  sub format_fragment_pod_simple_xhtml {
+    my ($self, $t) = @_;
+
+    # encode_entities {
+      $t =~ s/([$ents])/'&' . ($entities{$1} || sprintf '#x%X', ord $1) . ';'/ge;
+    # }
+
+    # idify {
+      for ($t) {
+          s/<[^>]+>//g;            # Strip HTML.
+          s/&[^;]+;//g;            # Strip entities.
+          s/^\s+//; s/\s+$//;      # Strip white space.
+          s/^([^a-zA-Z]+)$/pod$1/; # Prepend "pod" if no valid chars.
+          s/^[^a-zA-Z]+//;         # First char must be a letter.
+          s/[^-a-zA-Z0-9_:.]+/-/g; # All other chars must be valid.
+          s/[-:.]+$//;             # Strip trailing punctuation.
+      }
+    # }
+
+    return $t;
+  }
+}
+
+
+sub format_fragment_pod_simple_html {
+  my ($self, $section) = @_;
+
+  # From Pod::Simple::HTML 3.28.
+
+  # section_name_tidy {
+    $section =~ s/^\s+//;
+    $section =~ s/\s+$//;
+    $section =~ tr/ /_/;
+    $section =~ tr/\x00-\x1F\x80-\x9F//d if 'A' eq chr(65); # drop crazy characters
+
+    #$section = $self->unicode_escape_url($section);
+      # unicode_escape_url {
+      $section =~ s/([^\x00-\xFF])/'('.ord($1).')'/eg;
+        #  Turn char 1234 into "(1234)"
+      # }
+
+    $section = '_' unless length $section;
+    return $section;
+  # }
+}
+
+
+sub format_fragment_metacpan { shift->format_fragment_pod_simple_xhtml(@_); }
+sub format_fragment_sco      { shift->format_fragment_pod_simple_html(@_);  }
+
 1;
 
 __END__
@@ -418,7 +620,7 @@ __END__
 org> Aristotle Pagaltzis Randy Stauner ACKNOWLEDGEMENTS ATARASHI motemen
 moznion Graham Ollis Peter Vereshagin Yasutaka <aristotle@cpan.org>
 <plicease@cpan.org> <veresc@cpan.org> <rthompson@cpan.org> <yakex@cpan.org>
-<motemen@cpan.org> <moznion@cpan.org> textblock cpan testmatrix url
+<motemen@cpan.org> <moznion@cpan.org> textblock html cpan testmatrix url
 annocpan anno bugtracker rt cpants kwalitee diff irc mailto metadata
 placeholders metacpan
 
@@ -428,7 +630,7 @@ Pod::Markdown - Convert POD to Markdown
 
 =head1 VERSION
 
-version 1.401
+version 1.500
 
 =head1 SYNOPSIS
 
@@ -445,25 +647,27 @@ Literal characters in Pod that are special in Markdown
 
 =head1 METHODS
 
-=head2 perldoc_url_prefix
-
-Returns the url prefix in use (after resolving shortcuts to urls).
-
-=head2 man_url_prefix
-
-Returns the url prefix in use for man pages.
-
 =head2 new
 
+  Pod::Markdown->new(%options);
+
 The constructor accepts the following named arguments:
 
 =over 4
 
 =item *
 
+C<man_url_prefix>
+
+Alters the man page urls that are created from C<< LE<lt>E<gt> >> codes.
+
+The default is C<http://man.he.net/man>.
+
+=item *
+
 C<perldoc_url_prefix>
 
-This alters the perldoc urls that are created from C<< LE<lt>E<gt> >> codes.
+Alters the perldoc urls that are created from C<< LE<lt>E<gt> >> codes.
 Can be:
 
 =over 4
@@ -488,19 +692,128 @@ The default is C<metacpan>.
 
 =item *
 
-C<man_url_prefix>
+C<perldoc_fragment_format>
 
-This alters the man page urls that are created from C<< LE<lt>E<gt> >> codes.
+Alters the format of the url fragment for any C<< LE<lt>E<gt> >> links
+that point to a section of an external document (C<< L<name/section> >>).
+The default will be chosen according to the destination L</perldoc_url_prefix>.
+Alternatively you can specify one of the following:
 
-The default is C<http://man.he.net/man>.
+=over 4
+
+=item *
+
+C<metacpan>
+
+=item *
+
+C<sco>
+
+=item *
+
+C<pod_simple_xhtml>
+
+=item *
+
+C<pod_simple_html>
+
+=item *
+
+A code ref
 
 =back
 
+The code ref can expect to receive two arguments:
+the parser object (C<$self>) and the section text.
+For convenience the topic variable (C<$_>) is also set to the section text:
+
+  perldoc_fragment_format => sub { s/\W+/-/g; }
+
+=item *
+
+C<markdown_fragment_format>
+
+Alters the format of the url fragment for any C<< LE<lt>E<gt> >> links
+that point to an internal section of this document (C<< L</section> >>).
+
+Unfortunately the format of the id attributes produced
+by whatever system translates the markdown into html is unknown at the time
+the markdown is generated so we do some simple clean up.
+
+B<Note:> C<markdown_fragment_format> and C<perldoc_fragment_format> accept
+the same values: a (shortcut to a) method name or a code ref.
+
+=back
+
+=head2 man_url_prefix
+
+Returns the url prefix in use for man pages.
+
+=head2 perldoc_url_prefix
+
+Returns the url prefix in use (after resolving shortcuts to urls).
+
+=head2 perldoc_fragment_format
+
+Returns the coderef or format name used to format a url fragment
+to a section in an external document.
+
+=head2 markdown_fragment_format
+
+Returns the coderef or format name used to format a url fragment
+to an internal section in this document.
+
 =head2 as_markdown
 
 Returns the parsed POD as Markdown. Takes named arguments. If the C<with_meta>
 argument is given a positive value, meta tags are generated as well.
 
+=head2 format_man_url
+
+Used internally to create a url (using L</man_url_prefix>)
+from a string like C<man(1)>.
+
+=head2 format_perldoc_url
+
+    # With $name and section being the two parts of L<name/section>.
+    my $url = $parser->format_perldoc_url($name, $section);
+
+Used internally to create a url from
+the name (of a module or script)
+and a possible section (heading).
+
+The format of the url fragment (when pointing to a section in a document)
+varies depending on the destination url
+so L</perldoc_fragment_format> is used (which can be customized).
+
+If the module name portion of the link is blank
+then the section is treated as an internal fragment link
+(to a section of the generated markdown document)
+and L</markdown_fragment_format> is used (which can be customized).
+
+=head2 format_fragment_markdown
+
+Format url fragment for an internal link
+by replacing non-word characters with dashes.
+
+=head2 format_fragment_pod_simple_xhtml
+
+Format url fragment like L<Pod::Simple::XHTML/idify>.
+
+=head2 format_fragment_pod_simple_html
+
+Format url fragment like L<Pod::Simple::HTML/section_name_tidy>.
+
+=head2 format_fragment_metacpan
+
+Format fragment for L<metacpan.org>
+(uses L</format_fragment_pod_simple_xhtml>).
+
+=head2 format_fragment_sco
+
+Format fragment for L<search.cpan.org>
+(uses L</format_fragment_pod_simple_html>).
+
 =for Pod::Coverage format_header
 initialize
 command interior_sequence textblock verbatim
